from dataclasses import dataclass

@dataclass
class Speaker:
    base_name: str=None # default name
    name: str=None # User given name

    @property
    def label(self):
        return self.name or self.base_name



from transcriptor.alternatives import Alternative
from transcriptor.speakers import Speaker
from transcriptor.markers import Marker

import time
import logging
import typing
import more_itertools


class Job:
    def __init__(
            self,
            name: str,
            base_text: str,
            markers: typing.List[Marker],
            transcription: typing.Dict[typing.Any, typing.Any],
            alternatives: typing.List[Alternative],
            speakers: typing.List[Speaker],
            ):

        self.name = name
        self.base_text = base_text # default transcription as text from rendering service
        self.speakers = speakers
        self.markers = []
        for marker in markers:
            speaker = None
            if marker['speaker']:
                for x, s in enumerate(self.speakers):
                    if s.base_name == marker['speaker']:
                        speaker = self.speakers[x]
                        break

            m = Marker(
                    speaker=speaker,
                    start_time=marker['start_time'],
                    end_time=marker['end_time'],
                    )

            self.markers.append(m)

        self.transcription = transcription # original transcription object pre-processed
        self.alternatives = alternatives

    def get_text_at_marker(self, marker) -> str:
        """Given a marker index, return the text. Formatted for that object"""
        marker_alternatives = []

        for item in self.alternatives:

            # punctuation items do not have start/end times
            if marker_alternatives and item._type == 'punctuation':
                marker_alternatives.append(item.content)

            elif item.start_time >= marker.start_time and item.end_time <= marker.end_time:

                # Add space if not the first pronunciation
                if marker_alternatives:
                    content = f' {item.content}'

                else:
                    content = item.content

                marker_alternatives.append(content)

            elif item.end_time > marker.end_time:
                break # to stop processing once you have left the range.

        return ''.join(marker_alternatives)


    def as_text(
            self, separator: str='\n\n', text_separator: str=':\n\n',
            has_speaker: bool=True, has_timestamp: bool=True,
            time_format="%H:%M:%S",
            ) -> str:

        transcription_text = []

        for marker in self.markers:
            text = self.get_text_at_marker(marker)
            speaker = ''
            start_time = ''

            if has_speaker and marker.speaker:
                speaker = marker.speaker.label + ' '

            if has_timestamp:
                start_time = time.strftime(
                        time_format,
                        time.gmtime(marker.start_time,
                            ),
                    )

            transcription_text.append(
                f'{speaker}{start_time}{text_separator}{text}',
                )


        return separator.join(transcription_text)



from transcriptor.job import Job
from transcriptor.markers import Marker
from transcriptor.speakers import Speaker
from transcriptor.alternatives import Alternative

import boto3
import more_itertools
import requests
import typing

transcribe = boto3.client('transcribe')

def add_speaker(speaker_index: int) -> Speaker:
    """Create a speaker object from one of the labels in results['speaker_labels']"""
    return Speaker(
            base_name=f'spk_{speaker_index}',
            )


def add_marker(
        segment: typing.Sequence,
        *,
        has_speakers:bool=False,
        start_time=0.0,
        end_time=0.0,
        ) -> typing.Dict:
    """Create a Marker object using the speaker time_stamps OR the defined
    item_segments if speakers is False"""

    if has_speakers:
        speaker = segment['speaker_label']
        start_time = float(segment.get('start_time', start_time))
        end_time = float(segment.get('end_time', end_time))

    else:
        speaker = None
        start_time = float(segment[0]['start_time'])

        if len(segment) >= 2:
            end_time = float(segment[-2]['end_time'])

        else:
            end_time = float(segment[-1]['end_time'])

    return {
            'speaker': speaker,
            'start_time': start_time,
            'end_time': end_time,
           }


def add_alternative(segment:typing.Dict, start_time=0.0, end_time=0.0) -> Alternative:
        return Alternative(
                content = segment['alternatives'][0]['content'],
                _type = segment['type'],
                confidence = float(segment['alternatives'][0]['confidence']),
                start_time = float(segment.get('start_time', start_time)),
                end_time = float(segment.get('end_time', end_time)),
                )


def from_job(job_name: str) -> Job:
    """Create a Job Object based on the TranscriptiobJobName"""
    job = transcribe.get_transcription_job(TranscriptionJobName=job_name)
    return from_uri(job['Transcript']['TranscriptFileUri'])


def from_uri(uri) -> Job:
    """Create a Job Object based on the TranscriptFileUri"""
    response = requests.get(uri)
    response.raise_for_status()

    return from_json(response.json())

def from_json(transcription) -> Job:
    """Create a Job Object when given an Amazon JSON Object"""
    if 'speaker_labels' in transcription['results']:
        labels = transcription['results']['speaker_labels']
        speakers = [add_speaker(x) for x in range(labels['speakers'])]
        markers = [add_marker(x, has_speakers=True) for x in labels['segments']]

    else:
        segments = transcription['results']['items']
        speakers = []
        items_segments = more_itertools.split_when(
            segments, lambda x,y: x['alternatives'][0]['content'] in ['.', '?', '!'],
        )
        markers = []
        start_time = 0.0
        end_time = 0.0

        for marker_object in items_segments:
            marker = add_marker(
                    marker_object,
                    start_time=start_time,
                    end_time=end_time,
            )
            markers.append(marker)
            start_time = marker['start_time']
            end_time = marker['end_time']

    alternatives = []
    start_time = 0.0
    end_time = 0.0

    for alternative_object in transcription['results']['items']:
        alternative = add_alternative(
            alternative_object,
            start_time=start_time,
            end_time=end_time,
            )
        alternatives.append(alternative)
        start_time = alternative.start_time
        end_time = alternative.end_time

    return Job(
            base_text = transcription['results']['transcripts'][0]['transcript'],
            alternatives = alternatives,
            name = transcription['jobName'],
            transcription=transcription,
            speakers = speakers,
            markers = markers,
            )



from setuptools import setup, find_packages

with open("README.md") as filename:
    long_description = filename.read()


setup(
        name='transcriptor',
        version='2020.5.14',
        description='A wrapper for transcription results.',
        long_description=long_description,
        long_description_content_type='text/markdown',
        url='https://github.com/kjaymiller/transcriptor',
        author='Jay Miller',
        author_email='kjaymiller@gmail.com',
        license='MIT',
        packages=find_packages(),
        install_requires=[
            'more-itertools',
            'requests',
            ],
        zip_safe=False,
        )
